#include <avr/io.h>
#include <util/delay.h>

#include "common.h"
#include "extern.h"

void wait_ms(uint16_t t){
	while(t--){_delay_ms(1);}
}

void set_segment(uint8_t msb, uint8_t lsb){
	SEG_HIGH = msb;
	SEG_LOW  = lsb;
}

void set_dot(uint8_t pwm){
	 SEG_DOT_PWM = pwm;
}

void on_sw_led(void){
	SW_LED_PORT |= _BV(SW_LED);
}

void off_sw_led(void){
	SW_LED_PORT &= ~_BV(SW_LED);
}

void toggle_sw_led(void){
	SW_LED_PORT ^= _BV(SW_LED);
}

void on_gps_sw_led(void){
	GPS_SW_LED_PORT |= _BV(GPS_SW_LED);
}

void off_gps_sw_led(void){
	GPS_SW_LED_PORT &= ~_BV(GPS_SW_LED);
}

void toggle_gps_sw_led(void){
	GPS_SW_LED_PORT ^= _BV(GPS_SW_LED);
}

void on_gps_status_led(void){
	GPS_STATUS_LED_PORT |= _BV(GPS_STATUS_LED);
}

void off_gps_status_led(void){
	GPS_STATUS_LED_PORT &= ~_BV(GPS_STATUS_LED);
}

void toggle_gps_status_led(void){
	GPS_STATUS_LED_PORT ^= _BV(GPS_STATUS_LED);
}

uint8_t get_enc_1_B(void){
	return (ENC_1_B_PORT & _BV(ENC_1_B));
}

uint8_t get_enc_1_SW(void){
	return (ENC_1_SW_PORT & _BV(ENC_1_SW));
}

uint8_t get_enc_2_B(void){
	return (ENC_2_B_PORT & _BV(ENC_2_B));
}

uint8_t get_enc_2_SW(void){
	return (ENC_2_SW_PORT & _BV(ENC_2_SW));
}

void start_gps(void){
	GPS_PS_PORT |= _BV(GPS_PS);
	on_gps_sw_led();
	gps_count = GPS_COUNT;
}

void stop_gps(void){
	GPS_PS_PORT &= ~_BV(GPS_PS);
	off_gps_sw_led();
	gps_count = GPS_COUNT;
}

void toggle_gps(uint8_t val){
	if(val == ENABLE){
		GPS_PS_PORT |= _BV(GPS_PS);
		on_gps_sw_led();
	}else{
		GPS_PS_PORT &= ~_BV(GPS_PS);
		off_gps_sw_led();
	}
	gps_count = GPS_COUNT;
}

// maximum brightness
void max_brightness(void){
	uint8_t i;

	duty = DUTY_MAX;
	for(i=0; i<DIGIT_MAX; i++){
		digit_brightness[i] = DUTY_MAX;
	}
	colon_brightness[0] = PWM_MAX; // control colon brightness
	colon_brightness[1] = PWM_MAX; // store colon brightness
}

// return to normal brightness
void normal_brightness(void){
	uint8_t i;

	if(duty > DUTY_MAX || duty < DUTY_MIN){duty = DUTY_MAX;}
	for(i=0; i<DIGIT_MAX; i++){
		digit_brightness[i] = duty;
	}
	if(colon_brightness[1] > PWM_MAX || colon_brightness[1] < PWM_MIN){
		colon_brightness[1] = PWM_MAX;
	}
	colon_brightness[0] = colon_brightness[1];
	if(gps_en == ENABLE){on_gps_sw_led();}
	if(gps_count <= GPS_COUNT){on_gps_status_led();}

	limit_flag = DISABLE;
}

// limit all brightnes
void limit_brightness(void){
	uint8_t i;

	for(i=0; i<DIGIT_MAX; i++){
		digit_brightness[i] = DUTY_MIN;
	}
	colon_brightness[0] = PWM_LOW;
	off_sw_led();
	off_gps_sw_led();
	off_gps_status_led();

	limit_flag = ENABLE;
}

// change digit brightness temporarily
void set_brightness(uint8_t num, uint8_t val){
	if(num >= 0 && num < DIGIT_MAX){
		if(val > DUTY_MAX || val < DUTY_MIN){val = DUTY_MAX;}
		digit_brightness[num] = val;
	}
}

// change digit brightness
void set_source_brightness(uint8_t val){
	if(val > DUTY_MAX || val < DUTY_MIN){val = DUTY_MAX;}
	duty = val;
}

// change colon brightness temporarily
void set_colon_brightness(uint8_t val){
	if(val > PWM_MAX || val < PWM_MIN){val = PWM_MAX;}
	colon_brightness[0] = val;
}

// change colon brightness
void set_colon_source_brightness(uint8_t val){
	if(val > PWM_MAX || val < PWM_MIN){val = PWM_MAX;}
	colon_brightness[1] = val;
}

// turn off all the LED
void off_all_LED(void){
	uint8_t i;

	for(i=0; i<DIGIT_MAX; i++){
		digit_main[i] = SUPRESS;
		digit_dot[i] = PWM_MIN;
	}
	colon_brightness[0] = 0;
	off_sw_led();
	off_gps_sw_led();
	off_gps_status_led();

	limit_flag = ENABLE;
}

uint8_t get_day_char1(uint8_t val){
	switch(val){
		case 1: // sun 'S'
			return 'S' - FONT_OFFSET;

		case 2: // mon 'M'
			return 'M' - FONT_OFFSET;

		case 3: // tue 'T'
			return 'T' - FONT_OFFSET;

		case 4: // wed 'W'
			return 'W' - FONT_OFFSET;

		case 5: // thu 'T'
			return 'T' - FONT_OFFSET;

		case 6: // fri 'F'
			return 'F' - FONT_OFFSET;

		case 7: // sat 'S'
			return 'S' - FONT_OFFSET;

		default: // NG 'N"
			return 'N' - FONT_OFFSET;
	}

	return 'N' - FONT_OFFSET; // NG 'N'
}

uint8_t get_day_char2(uint8_t val){
	switch(val){
		case 1: // sun 'U'
			return 'U' - FONT_OFFSET;

		case 2: // mon 'O'
			return 'O' - FONT_OFFSET;

		case 3: // tue 'U'
			return 'U' - FONT_OFFSET;

		case 4: // wed 'E'
			return 'E' - FONT_OFFSET;

		case 5: // thu 'H'
			return 'H' - FONT_OFFSET;

		case 6: // fri 'R'
			return 'R' - FONT_OFFSET;

		case 7: // sat 'A'
			return 'A' - FONT_OFFSET;

		default: // NG 'G"
			return 'G' - FONT_OFFSET;
	}

	return 'G' - FONT_OFFSET; // NG 'N'
}

uint8_t chk_leap_year(void){
	uint16_t year;

	year = century * 100;
	year += (rtc_year >> 4) * 10 + (rtc_year & 0b00001111);

	if(((year % 4) == 0 && (year % 100) != 0) || (year % 400) == 0){
		return 29;
	}
	return 28;
}

uint8_t get_max_date(void){
	uint8_t month;

	month = ((rtc_month & 0b00010000) >> 4) * 10 + (rtc_month & 0b00001111);

	switch(month){
	case 1:
		return 31;

	case 2:
		return chk_leap_year();

	case 3:
		return 31;

	case 4:
		return 30;

	case 5:
		return 31;

	case 6:
		return 30;

	case 7:
		return 31;

	case 8:
		return 31;

	case 9:
		return 30;

	case 10:
		return 31;

	case 11:
		return 30;

	case 12:
		return 31;

	default:
		return 31;
	}

	return 31;
}
