#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/pgmspace.h>

#include "ad.h"
#include "common.h"
#include "eeprom.h"
#include "extern.h"
#include "rtc.h"
#include "timer.h"
#include "xitoa.h"

void init_timer(void){
	// timer0 for digit
	// normal mode
	TIMSK |= _BV(OCIE0);
	TCCR0 = _BV(CS00); // 1/1
	//TCCR0 = _BV(CS01); // 1/8
	//TCCR0 = _BV(CS00) | _BV(CS01); // 1/64

	// timer1 pwm
	// enable pwm(16bit timer) for dot
	TCCR1A = _BV(COM1A1) | _BV(COM1B1) | _BV(COM1C1) | _BV(WGM10);
	TCCR1B = _BV(WGM12) | _BV(CS10);
	OCR1A = 0;
	OCR1B = 0;
	OCR1C = 0;

	// timer2
	// control pwm
	TIMSK |= _BV(OCIE2);
	//TCCR2 = _BV(CS20); // 1/1
	//TCCR2 = _BV(CS21); // 1/8
	//TCCR2 = _BV(CS20) | _BV(CS21); // 1/64
	TCCR2 = _BV(CS22); // 1/256

	// timer3 for SW interrupts(2ms:500Hz)
	ETIMSK |= _BV(OCIE3A);
	TCCR3A = 0;
	TCCR3B = _BV(WGM32) | _BV(CS30); // 1/1
	//TCCR3B = _BV(CS31); // 1/8
	//TCCR3B = _BV(CS31) | _BV(CS30); // 1/64
	//TCCR3B = _BV(CS32); // 1/256
	//TCCR3B = _BV(CS32) | _BV(CS30); // 1/1024
	OCR3A = 32000;
}

ISR(TIMER0_COMP_vect){
	PORTF |= _BV(digit);
	if(duty_count < digit_brightness[digit]){
		//set_segment(pgm_read_byte(&font_high[digit_main[digit]]), pgm_read_byte(&font_low[digit_main[digit]]));
		set_segment(font_high[digit_main[digit]], font_low[digit_main[digit]]);
		set_dot(digit_dot[digit]);
	}else{
		PORTF &= ~_BV(digit);
		set_segment(0, 0);
		set_dot(0);
	}
	duty_count++;
	if(duty_count > DUTY_CYCLE){
		duty_count = 0;
		digit++;
	}
	if(digit >= DIGIT_MAX){digit = 0;}
}

ISR(TIMER2_COMP_vect){
	if(mode == TIME_MODE || mode == DATE_MODE){
		switch(colon_mode){
		case COLON_PWM:
			if((rtc_sec & 0b00001111) % 2 == 0){
				if(L_COLON_PWM <= colon_brightness[0]){
					L_COLON_PWM++;
					L_COLON_PWM++;
					R_COLON_PWM++;
					R_COLON_PWM++;
				}
			}else{
				if(L_COLON_PWM >= PWM_MIN){
					L_COLON_PWM--;
					L_COLON_PWM--;
					R_COLON_PWM--;
					R_COLON_PWM--;
				}
			}
			break;

		case COLON_SW:
			if((rtc_sec & 0b00001111) % 2 == 0){
				L_COLON_PWM = colon_brightness[0];
				R_COLON_PWM = colon_brightness[0];
			}else{
				L_COLON_PWM = PWM_MIN;
				R_COLON_PWM = PWM_MIN;
			}
			break;

		case COLON_ON:
			L_COLON_PWM = colon_brightness[0];
			R_COLON_PWM = colon_brightness[0];
			break;

		case COLON_OFF:
			L_COLON_PWM = PWM_MIN;
			R_COLON_PWM = PWM_MIN;
			break;

		default:
			break;
		}
	}else if(config_mode == CONFIG_BRIGHTNESS){
		L_COLON_PWM = colon_brightness[0];
		R_COLON_PWM = colon_brightness[0];
	}else{
		L_COLON_PWM = PWM_MIN;
		R_COLON_PWM = PWM_MIN;
	}
}

// watch various, SW etc.
// SW input configure clock
ISR(TIMER3_COMPA_vect){
	// SW2
	static uint8_t PIND4_old = _BV(PD4); // PIND4 old
	static uint8_t PIND_4;               // removed chattering PIND4
	uint8_t PIND4_new;                   // PIND4 raw
	// SW3
	static uint8_t PIND5_old = _BV(PD5); // PIND5 old
	static uint8_t PIND_5;               // removed chattering PIND5
	uint8_t PIND5_new;                   // PIND5 raw
	// SW4
	static uint8_t PIND6_old = _BV(PD6); // PIND6 old
	static uint8_t PIND_6;               // removed chattering PIND6
	uint8_t PIND6_new;                   // PIND6 raw
	// SW5
	static uint8_t PIND7_old = _BV(PD7); // PIND7 old
	static uint8_t PIND_7;               // removed chattering PIND7
	uint8_t PIND7_new;                   // PIND7 raw
	// ENC1 SW
	static uint8_t PING1_old = _BV(PG1); // PING1 old
	static uint8_t PING_1;               // removed chattering PING1
	uint8_t PING1_new;                   // PING1 raw
	// ENC2 SW
	static uint8_t PING3_old = _BV(PG3); // PING3 old
	static uint8_t PING_3;               // removed chattering PING3
	uint8_t PING3_new;                   // PING1 raw

	uint8_t temp;
	static uint8_t enc[2];

	uint8_t temp_time = 0;
	static uint8_t temp_rtc_hour, temp_rtc_day;
	uint8_t buf[3] = {0};

	static uint8_t temp_gps_en = 0;
	static uint8_t temp_duty = 0;
	static uint8_t temp_light_sensor_en = 0;
	static uint16_t temp_light_sensor_th = 0;
	static uint8_t temp_light_sensor_sw = 0;
	static int8_t temp_gps_timezone = 0;

	// SW2
	PIND4_new = PIND & _BV(PD4);
	temp = (~(PIND4_old ^ PIND4_new) & (PIND_4 ^ PIND4_new)); // twice same & afterward is different
	PIND4_old = PIND4_new;
	PIND_4 ^= temp;
	// SW3
	PIND5_new = PIND & _BV(PD5);
	temp = (~(PIND5_old ^ PIND5_new) & (PIND_5 ^ PIND5_new)); // twice same & afterward is different
	PIND5_old = PIND5_new;
	PIND_5 ^= temp;
	// SW4
	PIND6_new = PIND & _BV(PD6);
	temp = (~(PIND6_old ^ PIND6_new) & (PIND_6 ^ PIND6_new)); // twice same & afterward is different
	PIND6_old = PIND6_new;
	PIND_6 ^= temp;
	// SW5
	PIND7_new = PIND & _BV(PD7);
	temp = (~(PIND7_old ^ PIND7_new) & (PIND_7 ^ PIND7_new)); // twice same & afterward is different
	PIND7_old = PIND7_new;
	PIND_7 ^= temp;
	// ENC1 SW
	PING1_new = PING & _BV(PG1);
	temp = (~(PING1_old ^ PING1_new) & (PING_1 ^ PING1_new)); // twice same & afterward is different
	PING1_old = PING1_new;
	PING_1 ^= temp;
	// ENC2 SW
	PING3_new = PING & _BV(PG3);
	temp = (~(PING3_old ^ PING3_new) & (PING_3 ^ PING3_new)); // twice same & afterward is different
	PING3_old = PING3_new;
	PING_3 ^= temp;

	// ENC1
	// http://elm-chan.org/docs/tec/te04.html
	temp = 0;
	if((PINE & _BV(PE5))){temp = 1;}
	if((PING & _BV(PG0))){temp += 2;}
	enc[0] = (enc[0] << 2) + temp;
	enc[0] &= 15;
	// ENC2
	temp = 0;
	if((PINE & _BV(PE6))){temp = 1;}
	if((PING & _BV(PG2))){temp += 2;}
	enc[1] = (enc[1] << 2) + temp;
	enc[1] &= 15;

	
	// SW2
	if((PIND_4 == 0) && (PIND4_old == _BV(PD4))){
		toggle_sw_led();
		if(config_mode == CONFIG_NONE){
			mode++;
			eeprom_busy_wait();
			eeprom_update_byte((uint8_t *)E_MODE, mode);
		}
	}
	// SW3
	if((PIND_5 == 0) && (PIND5_old == _BV(PD5))){
		toggle_sw_led();
		switch(mode){
		case TIME_MODE:
			if(config_mode == CONFIG_NONE){
				colon_mode++;
				eeprom_busy_wait();
				eeprom_update_byte((uint8_t *)E_COLON_MODE, colon_mode);
				if(colon_mode > COLON_OFF){colon_mode = 0;}
			}
			break;

		case DATE_MODE:
			if(config_mode == CONFIG_NONE){
				colon_mode++;
				eeprom_busy_wait();
				eeprom_update_byte((uint8_t *)E_COLON_MODE, colon_mode);
				if(colon_mode > COLON_OFF){colon_mode = 0;}
			}
			break;

		case YEAR_MODE:
			break;

		case BRIGHTNESS:
			break;

		case LIGHT_SENSOR_EN:
			break;

		case LIGHT_SENSOR_TH:
			break;

		case LIGHT_SENSOR_SW:
			break;

		case GPS_EN:
			break;

		case GPS_TZ:
			break;

		default:
			break;
		}
	}
	// SW4
	if((PIND_6 == 0) && (PIND6_old == _BV(PD6))){
		toggle_sw_led();
		switch(mode){
		case TIME_MODE:
			if(config_mode == CONFIG_NONE){
				if(rtc_hour & AMPM_FLAG){
					temp_time = ((rtc_hour & 0b0010000) >> 4) * 10 + (rtc_hour & 0b00001111);
					if(rtc_hour & PM_FLAG){ // AMPM @ PM to 24hr
						if(temp_time != 12){temp_time += 12;}
						buf[0] = temp_time / 10;
						buf[1] = temp_time - buf[0] * 10;
						rtc_hour = (buf[0] << 4) | buf[1];
						rtc_write(RTC_HOUR_ADDR, rtc_hour);
					}else{ // AMPM @ AM to 24 hr
						if(temp_time == 12){
							rtc_hour = AMPM_FLAG;
						}else{
							rtc_hour &= ~(AMPM_FLAG);
						}
						rtc_write(RTC_HOUR_ADDR, rtc_hour);
					}
				}else{ // 24hr to AMPM
					temp_time = ((rtc_hour & 0b0110000) >> 4) * 10 + (rtc_hour & 0b00001111);
					if(temp_time >= 12){ // to AMPM @ PM
						if(temp_time != 12){temp_time -= 12;}
						buf[0] = temp_time / 10;
						buf[1] = temp_time - buf[0] * 10;
						rtc_hour = (AMPM_FLAG | PM_FLAG) | (buf[0] << 4) | buf[1];
						rtc_write(RTC_HOUR_ADDR, rtc_hour);
					}else{ // to AMPM @ AM
						if(temp_time == 0){temp_time = 12;}
						rtc_hour |= AMPM_FLAG;
						rtc_write(RTC_HOUR_ADDR, rtc_hour);
					}
				}
			}
			break;

		case DATE_MODE:
			break;

		case YEAR_MODE:
			break;

		case BRIGHTNESS:
			break;

		case LIGHT_SENSOR_EN:
			break;

		case LIGHT_SENSOR_TH:
			break;

		case LIGHT_SENSOR_SW:
			break;

		case GPS_EN:
			break;

		case GPS_TZ:
			break;

		default:
			break;
		}
	}
	// SW5
	if((PIND_7 == 0) && (PIND7_old == _BV(PD7))){
		toggle_sw_led();
		if(config_mode != CONFIG_NONE){
			config_mode = CONFIG_NONE;
			normal_brightness();
		}
		temp_turnon_flag = 1;
	}
	// ENC1 SW
	if((PING_1 == 0) && (PING1_old == _BV(PG1))){
		toggle_sw_led();
		switch(mode){
		case TIME_MODE:
			if(config_mode == CONFIG_NONE){
				config_mode = CONFIG_HOUR;
				set_brightness(0, DUTY_MAX);
				set_brightness(1, DUTY_MAX);
				set_brightness(2, DUTY_MIN);
				set_brightness(3, DUTY_MIN);
				set_brightness(4, DUTY_MIN);
				set_brightness(5, DUTY_MIN);
				set_colon_brightness(PWM_MIN);
				temp_rtc_hour = rtc_hour;
			}else if(config_mode == CONFIG_HOUR){
				config_mode = CONFIG_NONE;
				normal_brightness();
				rtc_write(RTC_HOUR_ADDR, temp_rtc_hour);
			}
			break;

		case DATE_MODE:
			if(config_mode == CONFIG_NONE){
				config_mode = CONFIG_MONTH;
				set_brightness(0, DUTY_MAX);
				set_brightness(1, DUTY_MAX);
				set_brightness(2, DUTY_MIN);
				set_brightness(3, DUTY_MIN);
				set_brightness(4, DUTY_MIN);
				set_brightness(5, DUTY_MIN);
				set_colon_brightness(PWM_MIN);
			}else if(config_mode == CONFIG_MONTH){
				config_mode = CONFIG_NONE;
				normal_brightness();
				rtc_write(RTC_MONTH_ADDR, (digit_main[0] << 4) | digit_main[1]);
			}
			break;

		case YEAR_MODE:
			if(config_mode == CONFIG_NONE){
				config_mode = CONFIG_CENTURY;
				set_brightness(0, DUTY_MAX);
				set_brightness(1, DUTY_MAX);
				set_brightness(2, DUTY_MIN);
				set_brightness(3, DUTY_MIN);
				set_brightness(4, DUTY_MIN);
				set_brightness(5, DUTY_MIN);
				set_colon_brightness(PWM_MIN);
			}else if(config_mode == CONFIG_CENTURY){
				config_mode = CONFIG_NONE;
				century = digit_main[0] * 10 + digit_main[1];
				normal_brightness();
				eeprom_busy_wait();
				eeprom_update_byte((uint8_t *)E_CENTURY, century);
			}
			break;

		case BRIGHTNESS:
			if(config_mode == CONFIG_NONE){
				config_mode = CONFIG_BRIGHTNESS;
				temp_duty = duty;
				set_brightness(0, DUTY_MIN);
				set_brightness(1, DUTY_MIN);
				set_brightness(2, DUTY_MIN);
				set_brightness(3, DUTY_MIN);
				set_brightness(4, DUTY_MAX);
				set_brightness(5, DUTY_MAX);
				set_colon_brightness(PWM_MIN);
			}else if(config_mode == CONFIG_BRIGHTNESS){
				config_mode = CONFIG_NONE;
				set_source_brightness(temp_duty);
				set_colon_source_brightness(temp_duty * DUTY_PWM_AMP1 - DUTY_PWM_AMP2);
				normal_brightness();
				eeprom_busy_wait();
				eeprom_update_byte((uint8_t *)E_DUTY, duty);
				eeprom_busy_wait();
				eeprom_update_byte((uint8_t *)E_COLON_BRIGHTNESS, colon_brightness[1]);
			}
			break;

		case LIGHT_SENSOR_EN:
			if(config_mode == CONFIG_NONE){
				config_mode = CONFIG_LIGHT_SENSOR_EN;
				temp_light_sensor_en = light_sensor_en;
				set_brightness(0, DUTY_MIN);
				set_brightness(1, DUTY_MIN);
				set_brightness(2, DUTY_MIN);
				set_brightness(3, DUTY_MAX);
				set_brightness(4, DUTY_MAX);
				set_brightness(5, DUTY_MAX);
				set_colon_brightness(PWM_MIN);
			}else if(config_mode == CONFIG_LIGHT_SENSOR_EN){
				config_mode = CONFIG_NONE;
				light_sensor_en = temp_light_sensor_en;
				normal_brightness();
				eeprom_busy_wait();
				eeprom_update_byte((uint8_t *)E_LIGHT_SENSOR_EN, light_sensor_en);
			}
			break;

		case LIGHT_SENSOR_TH:
			if(config_mode == CONFIG_NONE){
				config_mode = CONFIG_LIGHT_SENSOR_TH;
				temp_light_sensor_th = light_sensor_th;
				set_brightness(0, DUTY_MIN);
				set_brightness(1, DUTY_MIN);
				set_brightness(2, DUTY_MIN);
				set_brightness(3, DUTY_MAX);
				set_brightness(4, DUTY_MAX);
				set_brightness(5, DUTY_MAX);
				set_colon_brightness(PWM_MIN);
			}else if(config_mode == CONFIG_LIGHT_SENSOR_TH){
				config_mode = CONFIG_NONE;
				light_sensor_th = temp_light_sensor_th;
				normal_brightness();
				eeprom_busy_wait();
				eeprom_update_word((uint16_t *)E_LIGHT_SENSOR_TH, light_sensor_th);
			}
			break;

		case LIGHT_SENSOR_SW:
			if(config_mode == CONFIG_NONE){
				config_mode = CONFIG_LIGHT_SENSOR_SW;
				temp_light_sensor_sw = light_sensor_sw;
				set_brightness(0, DUTY_MIN);
				set_brightness(1, DUTY_MIN);
				set_brightness(2, DUTY_MIN);
				set_brightness(3, DUTY_MAX);
				set_brightness(4, DUTY_MAX);
				set_brightness(5, DUTY_MAX);
				set_colon_brightness(PWM_MIN);
			}else if(config_mode == CONFIG_LIGHT_SENSOR_SW){
				config_mode = CONFIG_NONE;
				light_sensor_sw = temp_light_sensor_sw;
				normal_brightness();
				eeprom_busy_wait();
				eeprom_update_byte((uint8_t *)E_LIGHT_SENSOR_SW, light_sensor_sw);
			}
			break;

		case GPS_EN:
			if(config_mode == CONFIG_NONE){
				config_mode = CONFIG_GPS_EN;
				temp_gps_en = gps_en;
				set_brightness(0, DUTY_MIN);
				set_brightness(1, DUTY_MIN);
				set_brightness(2, DUTY_MIN);
				set_brightness(3, DUTY_MAX);
				set_brightness(4, DUTY_MAX);
				set_brightness(5, DUTY_MAX);
				set_colon_brightness(PWM_MIN);
			}else if(config_mode == CONFIG_GPS_EN){
				config_mode = CONFIG_NONE;
				gps_en = temp_gps_en;
				toggle_gps(gps_en);
				normal_brightness();
				eeprom_busy_wait();
				eeprom_update_byte((uint8_t *)E_GPS_EN, gps_en);
			}
			break;

		case GPS_TZ:
			if(config_mode == CONFIG_NONE){
				config_mode = CONFIG_GPS_TZ;
				temp_gps_timezone = gps_timezone;
				set_brightness(0, DUTY_MIN);
				set_brightness(1, DUTY_MIN);
				set_brightness(2, DUTY_MIN);
				set_brightness(3, DUTY_MAX);
				set_brightness(4, DUTY_MAX);
				set_brightness(5, DUTY_MAX);
				set_colon_brightness(PWM_MIN);
			}else if(config_mode == CONFIG_GPS_TZ){
				config_mode = CONFIG_NONE;
				gps_timezone = temp_gps_timezone;
				normal_brightness();
				eeprom_busy_wait();
				eeprom_update_byte((uint8_t *)E_GPS_TIMEZONE, gps_timezone);
			}
			break;

		default:
			break;
		}
	}
	// ENC2 SW
	if((PING_3 == 0) && (PING3_old == _BV(PG3))){
		toggle_sw_led();
		switch(mode){
		case TIME_MODE:
			if(config_mode == CONFIG_NONE){
				config_mode = CONFIG_MIN;
				set_brightness(0, DUTY_MIN);
				set_brightness(1, DUTY_MIN);
				set_brightness(2, DUTY_MAX);
				set_brightness(3, DUTY_MAX);
				set_brightness(4, DUTY_MIN);
				set_brightness(5, DUTY_MIN);
				set_colon_brightness(PWM_MIN);
				digit_main[4] = 0;
				digit_main[5] = 0;
			}else if(config_mode == CONFIG_MIN){
				config_mode = CONFIG_NONE;
				normal_brightness();
				rtc_write(RTC_SEC_ADDR, 0);
				rtc_write(RTC_MIN_ADDR, (digit_main[2] << 4) | digit_main[3]);
			}
			break;

		case DATE_MODE:
			if(config_mode == CONFIG_NONE){
				config_mode = CONFIG_DATE;
				set_brightness(0, DUTY_MIN);
				set_brightness(1, DUTY_MIN);
				set_brightness(2, DUTY_MAX);
				set_brightness(3, DUTY_MAX);
				set_brightness(4, DUTY_MIN);
				set_brightness(5, DUTY_MIN);
				set_colon_brightness(PWM_MIN);
			}else if(config_mode == CONFIG_DATE){
				config_mode = CONFIG_DAY;
				set_brightness(0, DUTY_MIN);
				set_brightness(1, DUTY_MIN);
				set_brightness(2, DUTY_MIN);
				set_brightness(3, DUTY_MIN);
				set_brightness(4, DUTY_MAX);
				set_brightness(5, DUTY_MAX);
				set_colon_brightness(PWM_MIN);
				temp_rtc_day = rtc_day;
			}else if(config_mode == CONFIG_DAY){
				config_mode = CONFIG_NONE;
				normal_brightness();
				rtc_write(RTC_DATE_ADDR, (digit_main[2] << 4) | digit_main[3]);
				rtc_write(RTC_DAY_ADDR, temp_rtc_day);
			}
			break;

		case YEAR_MODE:
			if(config_mode == CONFIG_NONE){
				config_mode = CONFIG_YEAR;
				set_brightness(0, DUTY_MIN);
				set_brightness(1, DUTY_MIN);
				set_brightness(2, DUTY_MAX);
				set_brightness(3, DUTY_MAX);
				set_brightness(4, DUTY_MIN);
				set_brightness(5, DUTY_MIN);
				set_colon_brightness(PWM_MIN);
			}else if(config_mode == CONFIG_YEAR){
				config_mode = CONFIG_NONE;
				normal_brightness();
				rtc_write(RTC_YEAR_ADDR, (digit_main[2] << 4) | digit_main[3]);
			}
			break;

		case BRIGHTNESS:
			break;

		case LIGHT_SENSOR_EN:
			break;

		case LIGHT_SENSOR_TH:
			break;

		case LIGHT_SENSOR_SW:
			break;

		case GPS_EN:
			break;

		case GPS_TZ:
			break;

		default:
			break;
		}
	}

	// ENC1
	switch(enc[0]){
		case 0x7: // increment
			toggle_sw_led();
			switch(mode){
			case TIME_MODE:
				if(config_mode == CONFIG_HOUR){
					temp_time = digit_main[0] * 10 + digit_main[1];
					if((temp_rtc_hour & AMPM_FLAG) && !(temp_rtc_hour & PM_FLAG)){ // AMPM mode and time is AM
						if(temp_time == 11){ // up to PM
							temp_time++;
							xsprintf((char *)buf, PSTR("%02d"), temp_time);
							buf[0] -= 48;
							buf[1] -= 48;
							temp_rtc_hour = (AMPM_FLAG | PM_FLAG) | (buf[0] << 4) | buf[1];
							digit_main[0] = buf[0];
							digit_main[1] = buf[1];
							digit_dot[0] = PWM_MIN;
							digit_dot[1] = PWM_MAX;
						}else{
							if(temp_time == 12){
								temp_time = 1;
							}else{
								temp_time++;
							}							
							xsprintf((char *)buf, PSTR("%02d"), temp_time);
							buf[0] -= 48;
							buf[1] -= 48;
							temp_rtc_hour = AMPM_FLAG | (buf[0] << 4) | buf[1];
							digit_main[0] = buf[0];
							digit_main[1] = buf[1];
						}
					}else if((temp_rtc_hour & AMPM_FLAG) && (temp_rtc_hour & PM_FLAG)){ // AMPM mode and time is PM
						if(temp_time == 11){ // up to AM
							temp_time++;
							xsprintf((char *)buf, PSTR("%02d"), temp_time);
							buf[0] -= 48;
							buf[1] -= 48;
							temp_rtc_hour = AMPM_FLAG | (buf[0] << 4) | buf[1];
							digit_main[0] = buf[0];
							digit_main[1] = buf[1];
							digit_dot[0] = PWM_MAX;
							digit_dot[1] = PWM_MIN;
						}else{
							if(temp_time == 12){
								temp_time = 1;
							}else{
								temp_time++;
							}
							xsprintf((char *)buf, PSTR("%02d"), temp_time);
							buf[0] -= 48;
							buf[1] -= 48;
							temp_rtc_hour = (AMPM_FLAG | PM_FLAG) | (buf[0] << 4) | buf[1];
							digit_main[0] = buf[0];
							digit_main[1] = buf[1];
						}
					}else{ // 24hr mode
						if(temp_time == 23){
							temp_time = 0;
						}else{
							temp_time++;
						}
						xsprintf((char *)buf, PSTR("%02d"), temp_time);
						buf[0] -= 48;
						buf[1] -= 48;
						temp_rtc_hour = (buf[0] << 4) | buf[1];
						digit_main[0] = buf[0];
						digit_main[1] = buf[1];
					}
				}
				break;

			case DATE_MODE:
				if(config_mode == CONFIG_MONTH){
					temp_time = digit_main[0] * 10 + digit_main[1];
					if(temp_time == 12){
						temp_time = 1;
					}else{
						temp_time++;
					}
					xsprintf((char *)buf, PSTR("%02d"), temp_time);
					buf[0] -= 48;
					buf[1] -= 48;
					digit_main[0] = buf[0];
					digit_main[1] = buf[1];
				}
				break;

			case YEAR_MODE:
				if(config_mode == CONFIG_CENTURY){
					temp_time = digit_main[0] * 10 + digit_main[1];
					if(temp_time == 99){
						temp_time = 0;
					}else{
						temp_time++;
					}
					xsprintf((char *)buf, PSTR("%02d"), temp_time);
					buf[0] -= 48;
					buf[1] -= 48;
					digit_main[0] = buf[0];
					digit_main[1] = buf[1];
				}
				break;

			case BRIGHTNESS:
				if(config_mode == CONFIG_BRIGHTNESS){
					if(temp_duty - DUTY_MIN == DUTY_MAX - DUTY_MIN){
						temp_duty = DUTY_MIN;
					}else{
						temp_duty++;
					}
					digit_main[4] = (temp_duty - DUTY_MIN) / 10;
					digit_main[5] = (temp_duty - DUTY_MIN) - digit_main[4] * 10;
					set_brightness(4, temp_duty);
					set_brightness(5, temp_duty);
					set_colon_brightness(temp_duty * DUTY_PWM_AMP1 - DUTY_PWM_AMP2);
				}
				break;

			case LIGHT_SENSOR_EN:
				if(config_mode == CONFIG_LIGHT_SENSOR_EN){
					temp_light_sensor_en ^= 1;
					if(temp_light_sensor_en == 1){
						digit_main[3] = SUPRESS;
						digit_main[4] = 'O' - FONT_OFFSET;
						digit_main[5] = 'N' - FONT_OFFSET;
					}else{
						digit_main[3] = 'O' - FONT_OFFSET;
						digit_main[4] = 'F' - FONT_OFFSET;
						digit_main[5] = 'F' - FONT_OFFSET;
					}
				}
				break;

			case LIGHT_SENSOR_TH:
				if(config_mode == CONFIG_LIGHT_SENSOR_TH){
					if(temp_light_sensor_th == LIGHT_SENSOR_TH_MAX){
						temp_light_sensor_th = LIGHT_SENSOR_TH_MIN;
					}else{
						temp_light_sensor_th++;
					}
					xsprintf((char *)buf, PSTR("%03d"), temp_light_sensor_th);
					buf[0] -= 48;
					buf[1] -= 48;
					buf[2] -= 48;
					digit_main[3] = buf[0];
					digit_main[4] = buf[1];
					digit_main[5] = buf[2];
				}
				break;

			case LIGHT_SENSOR_SW:
				if(config_mode == CONFIG_LIGHT_SENSOR_SW){
					temp_light_sensor_sw ^= 1;
					if(temp_light_sensor_sw == ENABLE){
						digit_main[3] = SUPRESS;
						digit_main[4] = 'O' - FONT_OFFSET;
						digit_main[5] = 'N' - FONT_OFFSET;
					}else{
						digit_main[3] = 'O' - FONT_OFFSET;
						digit_main[4] = 'F' - FONT_OFFSET;
						digit_main[5] = 'F' - FONT_OFFSET;
					}
				}
				break;

			case GPS_EN:
				if(config_mode == CONFIG_GPS_EN){
					temp_gps_en ^= 1;
					if(temp_gps_en == ENABLE){
						digit_main[3] = SUPRESS;
						digit_main[4] = 'O' - FONT_OFFSET;
						digit_main[5] = 'N' - FONT_OFFSET;
					}else{
						digit_main[3] = 'O' - FONT_OFFSET;
						digit_main[4] = 'F' - FONT_OFFSET;
						digit_main[5] = 'F' - FONT_OFFSET;
					}
				}
				break;

			case GPS_TZ:
				if(config_mode == CONFIG_GPS_TZ){
						if(temp_gps_timezone == GPS_TIMEZONE_MAX){
							temp_gps_timezone = GPS_TIMEZONE_MIN;
						}else{
							temp_gps_timezone++;
						}
						if(temp_gps_timezone >= 0){
							digit_main[3] = SUPRESS;
							digit_main[4] = temp_gps_timezone / 10;
							digit_main[5] = temp_gps_timezone - digit_main[4] * 10;
						}else{
							digit_main[3] = MINUS;
							digit_main[4] = (~(temp_gps_timezone) + 1) / 10;
							digit_main[5] = (~(temp_gps_timezone) + 1) - digit_main[4] * 10;
						}
				}
				break;

			default:
				break;
			}
			break;
		
		case 0xd: // decrement
			toggle_sw_led();
			switch(mode){
			case TIME_MODE:
				if(config_mode == CONFIG_HOUR){
					temp_time = digit_main[0] * 10 + digit_main[1];
					if((temp_rtc_hour & AMPM_FLAG) && !(temp_rtc_hour & PM_FLAG)){ // AMPM mode and time is AM
						if(temp_time == 12){ // up to PM
							temp_time--;
							xsprintf((char *)buf, PSTR("%02d"), temp_time);
							buf[0] -= 48;
							buf[1] -= 48;
							temp_rtc_hour = (AMPM_FLAG | PM_FLAG) | (buf[0] << 4) | buf[1];
							digit_main[0] = buf[0];
							digit_main[1] = buf[1];
							digit_dot[0] = PWM_MIN;
							digit_dot[1] = PWM_MAX;
						}else{
							if(temp_time == 1){
								temp_time = 12;
							}else{
								temp_time--;
							}							
							xsprintf((char *)buf, PSTR("%02d"), temp_time);
							buf[0] -= 48;
							buf[1] -= 48;
							temp_rtc_hour = AMPM_FLAG | (buf[0] << 4) | buf[1];
							digit_main[0] = buf[0];
							digit_main[1] = buf[1];
						}
					}else if((temp_rtc_hour & AMPM_FLAG) && (temp_rtc_hour & PM_FLAG)){ // AMPM mode and time is PM
						if(temp_time == 12){ // up to AM
							temp_time--;
							xsprintf((char *)buf, PSTR("%02d"), temp_time);
							buf[0] -= 48;
							buf[1] -= 48;
							temp_rtc_hour = AMPM_FLAG | (buf[0] << 4) | buf[1];
							digit_main[0] = buf[0];
							digit_main[1] = buf[1];
							digit_dot[0] = PWM_MAX;
							digit_dot[1] = PWM_MIN;
						}else{
							if(temp_time == 1){
								temp_time = 12;
							}else{
								temp_time--;
							}
							xsprintf((char *)buf, PSTR("%02d"), temp_time);
							buf[0] -= 48;
							buf[1] -= 48;
							temp_rtc_hour = (AMPM_FLAG | PM_FLAG) | (buf[0] << 4) | buf[1];
							digit_main[0] = buf[0];
							digit_main[1] = buf[1];
						}
					}else{ // 24hr mode
						if(temp_time == 0){
							temp_time = 23;
						}else{
							temp_time--;
						}
						xsprintf((char *)buf, PSTR("%02d"), temp_time);
						buf[0] -= 48;
						buf[1] -= 48;
						temp_rtc_hour = (buf[0] << 4) | buf[1];
						digit_main[0] = buf[0];
						digit_main[1] = buf[1];
					}
				}
				break;

			case DATE_MODE:
				if(config_mode == CONFIG_MONTH){
						temp_time = digit_main[0] * 10 + digit_main[1];
						if(temp_time == 1){
							temp_time = 12;
						}else{
							temp_time--;
						}
						xsprintf((char *)buf, PSTR("%02d"), temp_time);
						buf[0] -= 48;
						buf[1] -= 48;
						digit_main[0] = buf[0];
						digit_main[1] = buf[1];
				}
				break;

			case YEAR_MODE:
				if(config_mode == CONFIG_CENTURY){
						temp_time = digit_main[0] * 10 + digit_main[1];
						if(temp_time == 0){
							temp_time = 99;
						}else{
							temp_time--;
						}
						xsprintf((char *)buf, PSTR("%02d"), temp_time);
						buf[0] -= 48;
						buf[1] -= 48;
						digit_main[0] = buf[0];
						digit_main[1] = buf[1];
				}
				break;

			case BRIGHTNESS:
				if(config_mode == CONFIG_BRIGHTNESS){
					if(temp_duty - DUTY_MIN == DUTY_MIN - DUTY_MIN){
						temp_duty = DUTY_MAX;
					}else{
						temp_duty--;
					}
					digit_main[4] = (temp_duty - DUTY_MIN) / 10;
					digit_main[5] = (temp_duty - DUTY_MIN) - digit_main[4] * 10;
					set_brightness(4, temp_duty);
					set_brightness(5, temp_duty);
					set_colon_brightness(temp_duty * DUTY_PWM_AMP1 - DUTY_PWM_AMP2);
				}
				break;

			case LIGHT_SENSOR_EN:
				if(config_mode == CONFIG_LIGHT_SENSOR_EN){
					temp_light_sensor_en ^= 1;
					if(temp_light_sensor_en == ENABLE){
						digit_main[3] = SUPRESS;
						digit_main[4] = 'O' - FONT_OFFSET;
						digit_main[5] = 'N' - FONT_OFFSET;
					}else{
						digit_main[3] = 'O' - FONT_OFFSET;
						digit_main[4] = 'F' - FONT_OFFSET;
						digit_main[5] = 'F' - FONT_OFFSET;
					}
				}
				break;

			case LIGHT_SENSOR_TH:
				if(config_mode == CONFIG_LIGHT_SENSOR_TH){
					if(temp_light_sensor_th == LIGHT_SENSOR_TH_MIN){
						temp_light_sensor_th = LIGHT_SENSOR_TH_MAX;
					}else{
						temp_light_sensor_th--;
					}
					xsprintf((char *)buf, PSTR("%03d"), temp_light_sensor_th);
					buf[0] -= 48;
					buf[1] -= 48;
					buf[2] -= 48;
					digit_main[3] = buf[0];
					digit_main[4] = buf[1];
					digit_main[5] = buf[2];
				}
				break;

			case LIGHT_SENSOR_SW:
				if(config_mode == CONFIG_LIGHT_SENSOR_SW){
					temp_light_sensor_sw ^= 1;
					if(temp_light_sensor_sw == ENABLE){
						digit_main[3] = SUPRESS;
						digit_main[4] = 'O' - FONT_OFFSET;
						digit_main[5] = 'N' - FONT_OFFSET;
					}else{
						digit_main[3] = 'O' - FONT_OFFSET;
						digit_main[4] = 'F' - FONT_OFFSET;
						digit_main[5] = 'F' - FONT_OFFSET;
					}
				}
				break;

			case GPS_EN:
				if(config_mode == CONFIG_GPS_EN){
					temp_gps_en ^= 1;
					if(temp_gps_en == ENABLE){
						digit_main[3] = SUPRESS;
						digit_main[4] = 'O' - FONT_OFFSET;
						digit_main[5] = 'N' - FONT_OFFSET;
					}else{
						digit_main[3] = 'O' - FONT_OFFSET;
						digit_main[4] = 'F' - FONT_OFFSET;
						digit_main[5] = 'F' - FONT_OFFSET;
					}
				}
				break;

			case GPS_TZ:
				if(config_mode == CONFIG_GPS_TZ){
						if(temp_gps_timezone == GPS_TIMEZONE_MIN){
							temp_gps_timezone = GPS_TIMEZONE_MAX;
						}else{
							temp_gps_timezone--;
						}
						if(temp_gps_timezone >= 0){
							digit_main[3] = SUPRESS;
							digit_main[4] = temp_gps_timezone / 10;
							digit_main[5] = temp_gps_timezone - digit_main[4] * 10;
						}else{
							digit_main[3] = MINUS;
							digit_main[4] = (~(temp_gps_timezone) + 1) / 10;
							digit_main[5] = (~(temp_gps_timezone) + 1) - digit_main[4] * 10;
						}
				}
				break;

			default:
				break;
			}
			break;
		default:
			break;
	}
	// ENC2
	switch(enc[1]){
		case 0x7: // increment
			toggle_sw_led();
			switch(mode){
			case TIME_MODE:
				if(config_mode == CONFIG_MIN){
						temp_time = digit_main[2] * 10 + digit_main[3];
						if(temp_time == 59){
							temp_time = 0;
						}else{
							temp_time++;
						}
						xsprintf((char *)buf, PSTR("%02d"), temp_time);
						buf[0] -= 48;
						buf[1] -= 48;
						digit_main[2] = buf[0];
						digit_main[3] = buf[1];
				}
				break;

			case DATE_MODE:
				if(config_mode == CONFIG_DATE){ // date
						temp_time = digit_main[2] * 10 + digit_main[3];
						if(temp_time == get_max_date()){
							temp_time = 1;
						}else{
							temp_time++;
						}
						xsprintf((char *)buf, PSTR("%02d"), temp_time);
						buf[0] -= 48;
						buf[1] -= 48;
						digit_main[2] = buf[0];
						digit_main[3] = buf[1];
				}else if(config_mode == CONFIG_DAY){ // day
					if(temp_rtc_day == 7){
						temp_rtc_day = 1;
					}else{
						temp_rtc_day++;
					}
					digit_main[4] = get_day_char1(temp_rtc_day);
					digit_main[5] = get_day_char2(temp_rtc_day);
				}
				break;

			case YEAR_MODE:
				if(config_mode == CONFIG_YEAR){
						temp_time = digit_main[2] * 10 + digit_main[3];
						if(temp_time == 99){
							temp_time = 0;
						}else{
							temp_time++;
						}
						xsprintf((char *)buf, PSTR("%02d"), temp_time);
						buf[0] -= 48;
						buf[1] -= 48;
						digit_main[2] = buf[0];
						digit_main[3] = buf[1];
				}
				break;

			case BRIGHTNESS:
				break;

			case LIGHT_SENSOR_EN:
				break;

			case LIGHT_SENSOR_TH:
				break;

			case LIGHT_SENSOR_SW:
				break;

			case GPS_EN:
				break;

			case GPS_TZ:
				break;

			default:
				break;
			}
			break;
		
		case 0xd: // decrement
			toggle_sw_led();
			switch(mode){
			case TIME_MODE:
				if(config_mode == CONFIG_MIN){
						temp_time = digit_main[2] * 10 + digit_main[3];
						if(temp_time == 0){
							temp_time = 59;
						}else{
							temp_time--;
						}
						xsprintf((char *)buf, PSTR("%02d"), temp_time);
						buf[0] -= 48;
						buf[1] -= 48;
						digit_main[2] = buf[0];
						digit_main[3] = buf[1];
				}
				break;

			case DATE_MODE:
				if(config_mode == CONFIG_DATE){ // date
						temp_time = digit_main[2] * 10 + digit_main[3];
						if(temp_time == 1){
							temp_time = get_max_date();
						}else{
							temp_time--;
						}
						xsprintf((char *)buf, PSTR("%02d"), temp_time);
						buf[0] -= 48;
						buf[1] -= 48;
						digit_main[2] = buf[0];
						digit_main[3] = buf[1];
				}else if(config_mode == CONFIG_DAY){ // day
					if(temp_rtc_day == 1){
						temp_rtc_day = 7;
					}else{
						temp_rtc_day--;
					}
					digit_main[4] = get_day_char1(temp_rtc_day);
					digit_main[5] = get_day_char2(temp_rtc_day);
				}
				break;

			case YEAR_MODE:
				if(config_mode == CONFIG_YEAR){
						temp_time = digit_main[2] * 10 + digit_main[3];
						if(temp_time == 0){
							temp_time = 99;
						}else{
							temp_time--;
						}
						xsprintf((char *)buf, PSTR("%02d"), temp_time);
						buf[0] -= 48;
						buf[1] -= 48;
						digit_main[2] = buf[0];
						digit_main[3] = buf[1];
				}
				break;

			case BRIGHTNESS:
				break;

			case LIGHT_SENSOR_EN:
				break;

			case LIGHT_SENSOR_TH:
				break;

			case LIGHT_SENSOR_SW:
				break;

			case GPS_EN:
				break;

			case GPS_TZ:
				break;

			default:
				break;
			}
			break;

		default:
			break;
	}
}
